﻿using System.Globalization;
using CsvHelper;
using CsvHelper.TypeConversion;

namespace Drive.Common.Csv
{
    public static class CsvExtensions
    {
        public static void WriteToCsv<TSource>(
            this IEnumerable<TSource> source,
            string path,
            IEnumerable<Tuple<Type, ITypeConverter>>? converters = null,
            IEnumerable<Type>? maps = null,
            CultureInfo? culture = null)
        {
            using FileStream stream = new(path, FileMode.Create);

            source.WriteToCsv(stream, converters, maps, culture);
        }

        public static void WriteToCsv<TSource>(
            this IEnumerable<TSource> source,
            Stream stream,
            IEnumerable<Tuple<Type, ITypeConverter>>? converters = null,
            IEnumerable<Type>? maps = null,
            CultureInfo? culture = null)
        {
            converters ??= Array.Empty<Tuple<Type, ITypeConverter>>();
            maps ??= Array.Empty<Type>();
            culture ??= CultureInfo.CurrentCulture;

            using StreamWriter writer = new(stream);
            using CsvWriter csv = new(writer, culture);

            foreach (var map in maps)
                csv.Context.RegisterClassMap(map);

            foreach (var converter in converters)
                csv.Context.TypeConverterCache.AddConverter(converter.Item1, converter.Item2);

            csv.WriteRecords(source);

            csv.Flush();
            writer.Flush();
        }
    }
}
