﻿using CsvHelper;
using CsvHelper.Configuration;
using CsvHelper.TypeConversion;
using NodaTime;
using NodaTime.Text;

namespace Drive.Common.Csv
{
    public class LocalDateCsvTypeConverter :
        ITypeConverter
    {
        public object ConvertFromString(string? text, IReaderRow row, MemberMapData memberMapData)
        {
            ParseResult<LocalDate> result = LocalDatePattern.Iso.Parse(text ?? string.Empty);
            if (!result.Success)
                throw result.Exception;

            return result.Value;
        }

        public string ConvertToString(object? value, IWriterRow row, MemberMapData memberMapData)
        {
            if (value is not null && value is LocalDate date)
                return LocalDatePattern.Iso.Format(date);

            throw new ArgumentException($"{value} is not a LocalDate instance", nameof(value));
        }
    }
}
