﻿using System.Data;
using System.Data.Common;
using NHibernate.Dialect;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.Type;

namespace Drive.Common.Data
{
    public class EnumerationType<T, S>
        : PrimitiveType
        where T : Enumeration<T, S>
        where S : notnull, IComparable
    {
        public EnumerationType()
            : base(StorageType()) { }

        private static SqlType StorageType()
        {
            DbType storageType = DbType.Int32;
            if (typeof(S) == typeof(short))
                storageType = DbType.Int16;
            else if (typeof(S) == typeof(long))
                storageType = DbType.Int64;

            return new SqlType(storageType);
        }

        public override Type ReturnedClass
        {
            get { return typeof(T); }
        }

        public override string Name
        {
            get { return "enumeration"; }
        }

        public override Type PrimitiveClass
        {
            get { return typeof(S); }
        }

        public override object DefaultValue
        {
            get { return default(S)!; }
        }

        public override object Get(DbDataReader rs, int index, ISessionImplementor session)
        {
            object stored = rs[index];
            return Enumeration<T, S>.FromValue((S)stored);
        }

        public override object Get(DbDataReader rs, string name, ISessionImplementor session)
        {
            return Get(rs, rs.GetOrdinal(name), session);
        }

        public override void Set(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            IDataParameter parameter = cmd.Parameters[index];
            Enumeration<T, S> asEnumeration = (Enumeration<T, S>)value;

            parameter.Value = asEnumeration.Value;
        }

        public override string? ObjectToSQLString(object value, Dialect dialect)
        {
            return value?.ToString();
        }
    }


    public class EnumerationType<T>
        : EnumerationType<T, int>
        where T : Enumeration<T, int>
    { }
}
