﻿using System.Data;
using System.Data.Common;
using NHibernate.Dialect;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.Type;
using NodaTime;

namespace Drive.Common.Data
{
    public class LocalTimeType : PrimitiveType
    {
        public LocalTimeType()
            : base(SqlTypeFactory.Time) { }

        public override string Name
        {
            get { return "LocalTime"; }
        }

        public override Type ReturnedClass
        {
            get { return typeof(LocalTime); }
        }

        public override object DefaultValue
        {
            get { return default(LocalTime); }
        }

        public override Type PrimitiveClass
        {
            get { return typeof(LocalTime); }
        }

        public override object Get(DbDataReader rs, int index, ISessionImplementor session)
        {
            try
            {
                TimeSpan timeSpan = (TimeSpan)rs[index];
                return new LocalTime(timeSpan.Hours, timeSpan.Minutes, timeSpan.Seconds, timeSpan.Milliseconds);
            }
            catch (Exception ex)
            {
                throw new FormatException(string.Format("Input string '{0}' was not in the correct format.", rs[index]), ex);
            }
        }

        public override object Get(DbDataReader rs, string name, ISessionImplementor session)
        {
            return Get(rs, rs.GetOrdinal(name), session);
        }

        public override void Set(DbCommand cmd, object value, int index, ISessionImplementor session)
        {
            IDataParameter param = cmd.Parameters[index];
            if (value == null)
                param.Value = DBNull.Value;
            else
            {
                var localTime = (LocalTime)value;
                param.DbType = DbType.Time;
                param.Value = new TimeSpan(0, localTime.Hour, localTime.Minute, localTime.Second, localTime.Millisecond);
            }
        }

        public override string ObjectToSQLString(object value, Dialect dialect)
        {
            return ((LocalTime)value).ToString("'hh:mm:ss'", null);
        }

        public override bool IsEqual(object x, object y)
        {
            if (x == y)
                return true;
            if (x == null || y == null)
                return false;

            return ((LocalTime)x).Equals(y);
        }

        public override int GetHashCode(object x, ISessionFactoryImplementor factory)
        {
            var time = (LocalTime)x;
            var hashCode = 1;
            unchecked
            {
                hashCode = 31 * hashCode + time.Hour;
                hashCode = 31 * hashCode + time.Minute;
                hashCode = 31 * hashCode + time.Second;
            }
            return hashCode;
        }

        public override string? ToLoggableString(object val, ISessionFactoryImplementor factory)
        {
            return val is not null && val is LocalTime actual ? actual.ToString("'hh:mm:ss'", null) : null;
        }
    }
}

