﻿using NodaTime;

namespace Drive.Common.Domain
{
    public interface ILegacyAggregateRootRecord
    {
        int Id { get; }
        int CreatedById { get; }
        Instant CreatedAt { get; }
        Instant? ModifiedAt { get; set; }
    }

    public interface ILegacyAuditedAggregateRoot
    {
        int ModifiedById { get; set; }
    }

    public interface ILegacyAggregateRoot<TRecord> where TRecord : ILegacyAggregateRootRecord
    {
        IEnumerable<object> GetEvents();
        TRecord GetRecord();
        void Restore(TRecord record);
    }

    public abstract class LegacyAggregateRoot<TRecord> : ILegacyAggregateRoot<TRecord> where TRecord : ILegacyAggregateRootRecord
    {
        private readonly Dictionary<Type, Action<object>> _handlers = new();
        private readonly List<object> _events = new();

#pragma warning disable CS8618 // Restore() should be called at construction
        protected TRecord _record;
#pragma warning restore CS8618 

        protected void Register<TEvent>(Action<TEvent> when)
        {
            _handlers.Add(typeof(TEvent), evt => when((TEvent)evt));
        }

        protected void Raise(object evt)
        {
            _handlers[evt.GetType()](evt);
            _events.Add(evt);
        }

        public IEnumerable<object> GetEvents()
        {
            return _events;
        }

        public TRecord GetRecord()
        {
            return _record;
        }

        void ILegacyAggregateRoot<TRecord>.Restore(TRecord record)
        {
            _record = record;
        }
    }
}
