﻿using MailKit.Net.Smtp;
using MailKit.Security;
using Microsoft.Extensions.Options;
using MimeKit;

namespace Drive.Common.Email
{
    public class SmtpEmailSender :
        ISendEmail
    {
        private readonly EmailOptions _options;

        public SmtpEmailSender(
            IOptions<EmailOptions> options)
        {
            _options = options.Value;
        }

        public async Task Send(
            EmailRequest request)
        {
            using SmtpClient client = new();

            if (string.IsNullOrWhiteSpace(_options.SmtpUsername))
            {
                await client.ConnectAsync(_options.SmtpServer, _options.SmtpPort!.Value);
            }
            else
            {
                await client.ConnectAsync(_options.SmtpServer, _options.SmtpPort!.Value, SecureSocketOptions.StartTls);
                await client.AuthenticateAsync(_options.SmtpUsername, _options.SmtpPassword);
            }

            MimeMessage message = new();

            message.From.Add(new MailboxAddress(string.Empty, request.From));

            foreach (string email in request.To)
            {
                message.To.Add(new MailboxAddress(string.Empty, email));
            }

            foreach (string email in request.Cc)
            {
                message.Cc.Add(new MailboxAddress(string.Empty, email));
            }

            message.Subject = request.Subject;

            BodyBuilder builder = new();
            foreach (EmailRequest.Attachment attachment in request.Attachments)
            {
                MimeEntity image = builder.LinkedResources.Add("", attachment.Content);
                image.ContentId = attachment.ContentId;
            }
            builder.HtmlBody = request.Content;

            message.Body = builder.ToMessageBody();

            await client.SendAsync(message);
            await client.DisconnectAsync(true);
        }
    }
}
