﻿using System.Security.Claims;
using Drive.Common.Security;
using NServiceBus.Pipeline;

namespace Drive.Common.Messaging.Signing
{
    public class MessageSigningBehavior
        : Behavior<IOutgoingPhysicalMessageContext>
    {
        public const string StepId = "MessageSigningBehavior";

        private readonly MessageHasher _hasher;
        private readonly SigningTokenGenerator _tokenGenerator;

        public MessageSigningBehavior(
            MessageHasher hasher,
            SigningTokenGenerator tokenGenerator)
        {
            _hasher = hasher;
            _tokenGenerator = tokenGenerator;
        }

        public override Task Invoke(IOutgoingPhysicalMessageContext context, Func<Task> next)
        {
            string hash = _hasher.Hash(context.Body);
            ClaimsPrincipal claimsPrincipal = context.Extensions.Get<ClaimsPrincipal>();
            List<Claim> claims = new()
            {
                claimsPrincipal.FindFirst(ClaimTypes.Name)!,
                claimsPrincipal.FindFirst(DriveClaimTypes.UserId)!,
                claimsPrincipal.FindFirst(DriveClaimTypes.UserBranchId)!,
                claimsPrincipal.FindFirst(DriveClaimTypes.StaffId)!,
                new Claim(DriveClaimTypes.MessageHash, hash),
            };
            string token = _tokenGenerator.GenerateToken(claims);

            context.Headers.Add(HeaderConstants.SIGNING_TOKEN, token);

            return next();
        }
    }
}
