﻿using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;
using Drive.Common.Security;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.Tokens;

namespace Drive.Common.Messaging.Signing
{
    public class SigningTokenGenerator
    {
        private readonly MessageSigningOptions _options;

        public SigningTokenGenerator(
            IOptions<MessageSigningOptions> options)
        {
            _options = options.Value;
        }

        public string GenerateToken(IEnumerable<Claim> claims)
        {
            DateTime expiresAt = DateTime.Now.AddDays(1);
            DateTime notBefore = DateTime.UtcNow.AddMinutes(-1);

            byte[] rawSigningKey = Encoding.UTF8.GetBytes(_options.Key);
            string issuer = _options.Issuer;
            string audience = _options.Audience;

            SymmetricSecurityKey key = new(rawSigningKey)
            {
                KeyId = _options.Key
            };

            SecurityTokenDescriptor tokenDescriptor = new()
            {
                Subject = new ClaimsIdentity(claims, TokenValidationParameters.DefaultAuthenticationType),
                Audience = audience,
                Issuer = issuer,
                Expires = expiresAt,
                NotBefore = notBefore,
                SigningCredentials = new SigningCredentials(
                    key,
                    SecurityAlgorithms.HmacSha256Signature,
                    SecurityAlgorithms.Sha256Digest)
            };
            JwtSecurityTokenHandler tokenHandler = new();
            SecurityToken token = tokenHandler.CreateToken(tokenDescriptor);
            return tokenHandler.WriteToken(token);
        }
    }
}
