﻿using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;
using Drive.Common.Security;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.Tokens;

namespace Drive.Common.Messaging.Signing
{
    public class SigningTokenValidator
    {
        private readonly MessageSigningOptions _signingOptions;

        public SigningTokenValidator(
            IOptions<MessageSigningOptions> signingOptions)
        {
            _signingOptions = signingOptions.Value;
        }

        public ClaimsPrincipal Validate(string token)
        {
            byte[] rawSigningKey = Encoding.UTF8.GetBytes(_signingOptions.Key);
            SecurityKey sigingKey = new SymmetricSecurityKey(rawSigningKey)
            {
                KeyId = _signingOptions.Key
            };

            TokenValidationParameters validationParameters = new()
            {
                ValidAudience = _signingOptions.Audience,
                ValidIssuer = _signingOptions.Issuer,
                ValidateLifetime = false,
                IssuerSigningKey = sigingKey,
            };
            JwtSecurityTokenHandler tokenHandler = new();
            return tokenHandler.ValidateToken(token, validationParameters, out _);
        }
    }
}
