﻿using System.Transactions;

namespace Drive.Common.Query
{
    public class QueryDispatcher : IQueryDispatcher
    {
        private readonly IServiceProvider _serviceProvider;

        public QueryDispatcher(IServiceProvider serviceProvider)
        {
            _serviceProvider = serviceProvider ?? throw new ArgumentNullException(nameof(serviceProvider));
        }

        public TResult Dispatch<TParameter, TResult>(TParameter query)
        {
            TResult result;

            using TransactionScope scope = new(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.ReadUncommitted });

            IQueryHandler<TParameter, TResult>? handler = (IQueryHandler<TParameter, TResult>)_serviceProvider.GetService(typeof(IQueryHandler<TParameter, TResult>))!;
            result = handler.Retrieve(query);

            scope.Complete();

            return result;
        }

        public TResult Dispatch<TResult>()
        {
            TResult result;

            using TransactionScope scope = new(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.ReadUncommitted });

            IQueryHandler<TResult>? handler = (IQueryHandler<TResult>)_serviceProvider.GetService(typeof(IQueryHandler<TResult>))!;
            result = handler.Retrieve();

            scope.Complete();

            return result;
        }
    }
}
