﻿using System.Security.Claims;
using Microsoft.Extensions.Caching.Memory;
using NodaTime;

namespace Drive.Common.Web
{
    public interface ICacheClaimsPrincipals
    {
        ClaimsPrincipal? Get(string key);
        void Remove(string key);
        void Set(string key, ClaimsPrincipal principal, Instant expires);
    }


    public class ClaimsPrincipalCache :
        ICacheClaimsPrincipals
    {
        private readonly IMemoryCache _cache;

        public ClaimsPrincipalCache(
            IMemoryCache cache)
        {
            _cache = cache;
        }

        public ClaimsPrincipal? Get(string key)
        {
            _cache.TryGetValue(key.ToLower(), out ClaimsPrincipal? cachedPrincipal);

            return cachedPrincipal;
        }

        public void Remove(string key)
        {
            _cache.Remove(key.ToLower());
        }

        public void Set(string key, ClaimsPrincipal principal, Instant expireAt)
        {
            MemoryCacheEntryOptions options = new()
            {
                AbsoluteExpiration = expireAt.ToDateTimeOffset(),
            };
            _cache.Set(key.ToLower(), principal, options);
        }
    }
}
