﻿using NodaTime;

namespace Drive.Common
{
    public interface IClockProvider
    {
        IClock Current { get; }
        LocalDate Today { get; }
        ZonedDateTime Now { get; }
        Instant UtcNow { get; }
    }

    public class DriveClockProvider : IClockProvider
    {
        private readonly DateTimeZone _zone;

        public DriveClockProvider(IClock? clock = null)
        {
            Current = clock ?? SystemClock.Instance;
            _zone = DateTimeZoneProviders.Tzdb.GetSystemDefault(); // possibly refactor to same approach as clock parameter for providing a specific zone
        }

        public IClock Current { get; }

        public Instant UtcNow => Current.GetCurrentInstant();
        public ZonedDateTime Now => UtcNow.InZone(_zone);
        public LocalDate Today => Now.Date;
    }
}
