﻿using NodaTime;

namespace Drive.Common.Extensions
{
    public static class DateExtensions
    {
        public static LocalDate MonthBegin(this LocalDate date)
        {
            return new LocalDate(date.Year, date.Month, 1);
        }

        public static LocalDate MonthEnd(this LocalDate date)
        {
            return new LocalDate(date.Year, date.Month, DateTime.DaysInMonth(date.Year, date.Month));
        }

        public static LocalDate PreviousMonthEnd(this LocalDate date)
        {
            return date.PlusMonths(-1).MonthEnd();
        }

        public static LocalDate NextMonthEnd(this LocalDate date)
        {
            return date.PlusMonths(1).MonthEnd();
        }

        public static bool FallsInMonth(this LocalDate toTest, LocalDate month)
        {
            return toTest.Month == month.Month && toTest.Year == month.Year;
        }

        public static DateTime MonthBegin(this DateTime date)
        {
            return new DateTime(date.Year, date.Month, 1);
        }

        public static LocalDate GetNextWorkingDay(this LocalDate startDate, int numberOfDaysInFuture = 1)
        {
            int counter = 0;
            LocalDate nextDate = startDate;

            while (counter < numberOfDaysInFuture)
            {
                nextDate = nextDate.PlusDays(1);

                //if the date is either a sat or sun, move the date again
                if (nextDate.DayOfWeek == IsoDayOfWeek.Saturday) nextDate = nextDate.PlusDays(2);
                if (nextDate.DayOfWeek == IsoDayOfWeek.Sunday) nextDate = nextDate.PlusDays(1);

                counter++;
            }

            return nextDate;
        }

        public static LocalDate GetPreviousWorkingDay(this LocalDate date, int numberOfDaysToMove = 1)
        {
            int counter = 0;
            LocalDate nextDate = date;

            while (counter < numberOfDaysToMove)
            {
                nextDate = nextDate.Minus(Period.FromDays(1));

                //if the date is either a sat or sun, move the date again
                if (nextDate.DayOfWeek == IsoDayOfWeek.Saturday) nextDate = nextDate.Minus(Period.FromDays(1));
                if (nextDate.DayOfWeek == IsoDayOfWeek.Sunday) nextDate = nextDate.Minus(Period.FromDays(2));

                counter++;
            }

            return nextDate;
        }
    }
}
