﻿using System.Linq.Expressions;

namespace Drive.Radio.Services.Core.Extensions
{
    public static class LinqExtensions
    {
        #region
        public static IOrderedQueryable<TSource> GenericOrderByWithDirection<TSource>
           (this IQueryable<TSource> query,
           string propertyName, bool descending)
        {
            return GenericOrderByHelper(query, propertyName, descending, false);
        }

        public static IOrderedQueryable<TSource> GenericThenByWithDirection<TSource>
          (this IQueryable<TSource> query,
          string propertyName, bool descending)
        {
            return GenericOrderByHelper(query, propertyName, descending, true);
        }

        public static IOrderedQueryable<TSource> GenericOrderByHelper<TSource>
          (this IQueryable<TSource> query,
         Expression<Func<TSource,object>> propertyExpression, bool descending, bool thenBy)
        {

            var typeArgs = propertyExpression.Type.GetGenericArguments();

            var call = Expression.Call(typeof(Queryable), (!thenBy ? "OrderBy" : "ThenBy") + (descending ? "Descending" : string.Empty), typeArgs, query.Expression, propertyExpression);
            return (IOrderedQueryable<TSource>)query.Provider.CreateQuery<TSource>(call);
        }

        public static IOrderedQueryable<TSource> GenericOrderByHelper<TSource>
          (this IQueryable<TSource> query,
          string propertyName, bool descending, bool thenBy)
        {
           
            ParameterExpression param = Expression.Parameter(typeof(TSource), string.Empty); // I don't care about some naming
            var parts = propertyName.Split('.');

            Expression property = param;

            foreach (var part in parts)
            {
                property = Expression.PropertyOrField(property, part);
            }

            LambdaExpression sort = Expression.Lambda(property, param);
            var typeArgs = new[] { typeof(TSource), property.Type };

            var call = Expression.Call(typeof(Queryable), (!thenBy ? "OrderBy" : "ThenBy") + (descending ? "Descending" : string.Empty), typeArgs, query.Expression, Expression.Quote(sort));
            return (IOrderedQueryable<TSource>)query.Provider.CreateQuery<TSource>(call);
        }

        public static IOrderedQueryable<TSource> GenericOrderByDescending<TSource>
           (this IQueryable<TSource> query,
           string propertyName)
        {
            return GenericOrderByHelper(query, propertyName, true, false);
        }

        public static IOrderedQueryable<TSource> GenericOrderBy<TSource>
          (this IQueryable<TSource> query,
          string propertyName)
        {
            return GenericOrderByHelper(query, propertyName, false, false);
        }

        public static IOrderedQueryable<TSource> GenericThenBy<TSource>
         (this IQueryable<TSource> query,
         string propertyName)
        {
            return GenericOrderByHelper(query, propertyName, false, true);
        }

        public static IOrderedQueryable<TSource> GenericThenByDescending<TSource>
         (this IQueryable<TSource> query,
         string propertyName)
        {
            return GenericOrderByHelper(query, propertyName, true, true);
        }
        #endregion
    }
        
}
