﻿using Drive.Common.Data;
using Drive.Common.IntegrationTest.IntegrationDatabaseTest;
using FluentAssertions;
using NHibernate;
using NodaTime;
using Xunit;

namespace Drive.Common.IntegrationTest.Data
{
    [Collection(TestIntegrationDatabaseTestCollection.COLLECTION_NAME)]
    public class AggregateRootPersistorSuite
    {
        private TestIntegrationDatabaseTest _databaseTest;

        public AggregateRootPersistorSuite(TestIntegrationDatabaseTest databaseTest)
        {
            _databaseTest = databaseTest;
        }


        [Fact]
        public async Task modified_by_id_is_persisted()
        {
            //arrange
            using ISession session = _databaseTest.OpenSession();
            AggregateRootPersister persistor = new AggregateRootPersister(session, new DriveClockProvider());

            TestAggregateRootModel model = new TestAggregateRootModel()
            {
                CreatedAt = Instant.FromDateTimeUtc(DateTime.Now.ToUniversalTime()),
                CreatedById = 1,
                Id = Guid.NewGuid(),
                ModifiedAt = Instant.FromDateTimeUtc(DateTime.Now.ToUniversalTime()),
                ModifiedById = 2
            };

            DriveContext context = new DriveContext();
            context.DriveUserId = 3;

            //act
            using ITransaction transaction = session.BeginTransaction();
            {
                await persistor.Persist(
                    new TestAggregateRoot(model),
                    context);

                transaction.Commit();
            }

            //assert
            model.ModifiedById.Should().Be(3);

        }
    }
}