﻿using System.Text;
using CsvHelper.Configuration;
using CsvHelper.TypeConversion;
using Drive.Common.Csv;
using FluentAssertions;
using NodaTime;
using Xunit;

namespace Drive.Common.UnitTest
{
    public class CsvExtensionsSuite : IDisposable
    {
        [Fact]
        public void write_to_csv_with_path_creates_file()
        {
            int[] forTest = new[] { 1, 2, 3, 4 };
            string writeTo = @"C:\temp\testoutput.csv";

            forTest.WriteToCsv(writeTo);

            File.Exists(writeTo).Should().BeTrue();
        }

        [Fact]
        public void write_to_csv_with_stream_creates_correct_output()
        {
            var forTest = new[] { new { X = 1, Y = 1 }, new { X = 2, Y = 2 }, new { X = 3, Y = 3 } };
            using MemoryStream stream = new();

            forTest.WriteToCsv(stream);

            string result = Encoding.Default.GetString(stream.ToArray());
            result.Should().Be(string.Format("X,Y{0}1,1{0}2,2{0}3,3{0}", Environment.NewLine));
        }

        [Fact]
        public void write_to_csv_with_map()
        {
            Test[] data = new[] { new Test { Id = 1, Date = new LocalDate(2021, 5, 2), Total = 1234.1234m } };
            MemoryStream stream = new();

            data.WriteToCsv(stream, Enumerable.Empty<Tuple<Type, ITypeConverter>>(), new[] { typeof(TestMap) });

            string result = Encoding.Default.GetString(stream.ToArray());
            result.Should().Be(string.Format("Id,Date,Total{0}1,02-05-2021,1234.12{0}", Environment.NewLine));
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                File.Delete(@"C:\temp\testoutput.csv");
            }
        }

        private class Test
        {
            public int Id { get; set; }
            public LocalDate Date { get; set; }
            public decimal Total { get; set; }
        }

        private class TestMap
        : ClassMap<Test>
        {
            public TestMap()
            {
                Map(x => x.Id);
                Map(x => x.Date).TypeConverterOption.Format("dd-MM-yyyy");
                Map(x => x.Total).TypeConverterOption.Format("0.00");
            }
        }
    }
}
