﻿using Drive.Common.Extensions;
using FluentAssertions;
using NodaTime;
using Xunit;

namespace Drive.Common.UnitTest
{
    public class DateExtensionsSuite
    {
        [Fact]
        public void month_begin_converts_day_in_month_to_first_day_of_that_month()
        {
            //
            LocalDate today = new LocalDate(2015, 1, 15);

            //
            LocalDate monthEnd = today.MonthBegin();

            //
            monthEnd.Day.Should().Be(1);
            monthEnd.Month.Should().Be(1);
            monthEnd.Year.Should().Be(2015);
        }

        [Fact]
        public void month_end_converts_day_in_month_to_last_day_of_that_month()
        {
            //
            LocalDate today = new LocalDate(2015, 1, 1);

            //
            LocalDate monthEnd = today.MonthEnd();

            //
            monthEnd.Day.Should().Be(31);
            monthEnd.Month.Should().Be(1);
            monthEnd.Year.Should().Be(2015);
        }

        [Fact]
        public void month_end_converts_day_in_month_to_last_day_of_that_month_for_normal_february()
        {
            //
            LocalDate normalFebruary = new LocalDate(2015, 2, 1);

            //
            LocalDate monthEnd = normalFebruary.MonthEnd();

            //
            monthEnd.Day.Should().Be(28);
            monthEnd.Month.Should().Be(2);
            monthEnd.Year.Should().Be(2015);
        }

        [Fact]
        public void month_end_converts_day_in_month_to_last_day_of_that_month_for_leap_year()
        {
            //
            LocalDate leapYear = new LocalDate(2016, 2, 1);

            //
            LocalDate monthEnd = leapYear.MonthEnd();

            //
            monthEnd.Day.Should().Be(29);
            monthEnd.Month.Should().Be(2);
            monthEnd.Year.Should().Be(2016);
        }

        [Fact]
        public void next_working_day_from_friday_is_monday()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 11);

            //
            LocalDate nextWorkingDay = startDay.GetNextWorkingDay(1);

            //
            nextWorkingDay.Day.Should().Be(14);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }

        [Fact]
        public void next_working_day_3_days_from_friday_is_wed()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 11);

            //
            LocalDate nextWorkingDay = startDay.GetNextWorkingDay(3);

            //
            nextWorkingDay.Day.Should().Be(16);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }

        [Fact]
        public void next_working_day_0_days_from_friday_is_friday()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 11);

            //
            LocalDate nextWorkingDay = startDay.GetNextWorkingDay(0);

            //
            nextWorkingDay.Day.Should().Be(11);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }

        [Fact]
        public void previous_working_day_from_monday_is_friday()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 20);

            //
            LocalDate nextWorkingDay = startDay.GetPreviousWorkingDay();

            //
            nextWorkingDay.Day.Should().Be(18);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }

        [Fact]
        public void previous_working_day_0_days_from_monday_is_monday()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 20);

            //
            LocalDate nextWorkingDay = startDay.GetNextWorkingDay(0);

            //
            nextWorkingDay.Day.Should().Be(20);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }

        [Fact]
        public void previous_working_day_3_days_from_tuesday_is_thursday()
        {
            //
            LocalDate startDay = new LocalDate(2016, 3, 22);

            //
            LocalDate nextWorkingDay = startDay.GetPreviousWorkingDay(3);

            //
            nextWorkingDay.Day.Should().Be(17);
            nextWorkingDay.Month.Should().Be(3);
            nextWorkingDay.Year.Should().Be(2016);
        }
    }
}
